=head1 Name

Go::UV::Constant - Constants for libuv

=head1 Class Methods

=head2 UV_VERSION_MAJOR

C<static method UV_VERSION_MAJOR : int ();>

Returns the value of C<UV_VERSION_MAJOR>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_VERSION_MINOR

C<static method UV_VERSION_MINOR : int ();>

Returns the value of C<UV_VERSION_MINOR>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_VERSION_PATCH

C<static method UV_VERSION_PATCH : int ();>

Returns the value of C<UV_VERSION_PATCH>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_VERSION_IS_RELEASE

C<static method UV_VERSION_IS_RELEASE : int ();>

Returns the value of C<UV_VERSION_IS_RELEASE>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_VERSION_HEX

C<static method UV_VERSION_HEX : int ();>

Returns the value of C<UV_VERSION_HEX>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_VERSION_SUFFIX

C<static method UV_VERSION_SUFFIX : string ();>

Returns the value of C<UV_VERSION_SUFFIX>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_E2BIG

C<static method UV_E2BIG : int ();>

Returns the value of C<UV_E2BIG>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EACCES

C<static method UV_EACCES : int ();>

Returns the value of C<UV_EACCES>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EADDRINUSE

C<static method UV_EADDRINUSE : int ();>

Returns the value of C<UV_EADDRINUSE>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EADDRNOTAVAIL

C<static method UV_EADDRNOTAVAIL : int ();>

Returns the value of C<UV_EADDRNOTAVAIL>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EAFNOSUPPORT

C<static method UV_EAFNOSUPPORT : int ();>

Returns the value of C<UV_EAFNOSUPPORT>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EAGAIN

C<static method UV_EAGAIN : int ();>

Returns the value of C<UV_EAGAIN>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EAI_ADDRFAMILY

C<static method UV_EAI_ADDRFAMILY : int ();>

Returns the value of C<UV_EAI_ADDRFAMILY>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EAI_AGAIN

C<static method UV_EAI_AGAIN : int ();>

Returns the value of C<UV_EAI_AGAIN>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EAI_BADFLAGS

C<static method UV_EAI_BADFLAGS : int ();>

Returns the value of C<UV_EAI_BADFLAGS>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EAI_BADHINTS

C<static method UV_EAI_BADHINTS : int ();>

Returns the value of C<UV_EAI_BADHINTS>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EAI_CANCELED

C<static method UV_EAI_CANCELED : int ();>

Returns the value of C<UV_EAI_CANCELED>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EAI_FAIL

C<static method UV_EAI_FAIL : int ();>

Returns the value of C<UV_EAI_FAIL>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EAI_FAMILY

C<static method UV_EAI_FAMILY : int ();>

Returns the value of C<UV_EAI_FAMILY>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EAI_MEMORY

C<static method UV_EAI_MEMORY : int ();>

Returns the value of C<UV_EAI_MEMORY>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EAI_NODATA

C<static method UV_EAI_NODATA : int ();>

Returns the value of C<UV_EAI_NODATA>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EAI_NONAME

C<static method UV_EAI_NONAME : int ();>

Returns the value of C<UV_EAI_NONAME>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EAI_OVERFLOW

C<static method UV_EAI_OVERFLOW : int ();>

Returns the value of C<UV_EAI_OVERFLOW>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EAI_PROTOCOL

C<static method UV_EAI_PROTOCOL : int ();>

Returns the value of C<UV_EAI_PROTOCOL>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EAI_SERVICE

C<static method UV_EAI_SERVICE : int ();>

Returns the value of C<UV_EAI_SERVICE>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EAI_SOCKTYPE

C<static method UV_EAI_SOCKTYPE : int ();>

Returns the value of C<UV_EAI_SOCKTYPE>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EALREADY

C<static method UV_EALREADY : int ();>

Returns the value of C<UV_EALREADY>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EBADF

C<static method UV_EBADF : int ();>

Returns the value of C<UV_EBADF>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EBUSY

C<static method UV_EBUSY : int ();>

Returns the value of C<UV_EBUSY>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ECANCELED

C<static method UV_ECANCELED : int ();>

Returns the value of C<UV_ECANCELED>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ECHARSET

C<static method UV_ECHARSET : int ();>

Returns the value of C<UV_ECHARSET>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ECONNABORTED

C<static method UV_ECONNABORTED : int ();>

Returns the value of C<UV_ECONNABORTED>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ECONNREFUSED

C<static method UV_ECONNREFUSED : int ();>

Returns the value of C<UV_ECONNREFUSED>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ECONNRESET

C<static method UV_ECONNRESET : int ();>

Returns the value of C<UV_ECONNRESET>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EDESTADDRREQ

C<static method UV_EDESTADDRREQ : int ();>

Returns the value of C<UV_EDESTADDRREQ>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EEXIST

C<static method UV_EEXIST : int ();>

Returns the value of C<UV_EEXIST>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EFAULT

C<static method UV_EFAULT : int ();>

Returns the value of C<UV_EFAULT>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EFBIG

C<static method UV_EFBIG : int ();>

Returns the value of C<UV_EFBIG>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EHOSTUNREACH

C<static method UV_EHOSTUNREACH : int ();>

Returns the value of C<UV_EHOSTUNREACH>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EINTR

C<static method UV_EINTR : int ();>

Returns the value of C<UV_EINTR>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EINVAL

C<static method UV_EINVAL : int ();>

Returns the value of C<UV_EINVAL>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EIO

C<static method UV_EIO : int ();>

Returns the value of C<UV_EIO>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EISCONN

C<static method UV_EISCONN : int ();>

Returns the value of C<UV_EISCONN>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EISDIR

C<static method UV_EISDIR : int ();>

Returns the value of C<UV_EISDIR>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ELOOP

C<static method UV_ELOOP : int ();>

Returns the value of C<UV_ELOOP>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EMFILE

C<static method UV_EMFILE : int ();>

Returns the value of C<UV_EMFILE>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EMSGSIZE

C<static method UV_EMSGSIZE : int ();>

Returns the value of C<UV_EMSGSIZE>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ENAMETOOLONG

C<static method UV_ENAMETOOLONG : int ();>

Returns the value of C<UV_ENAMETOOLONG>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ENETDOWN

C<static method UV_ENETDOWN : int ();>

Returns the value of C<UV_ENETDOWN>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ENETUNREACH

C<static method UV_ENETUNREACH : int ();>

Returns the value of C<UV_ENETUNREACH>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ENFILE

C<static method UV_ENFILE : int ();>

Returns the value of C<UV_ENFILE>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ENOBUFS

C<static method UV_ENOBUFS : int ();>

Returns the value of C<UV_ENOBUFS>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ENODEV

C<static method UV_ENODEV : int ();>

Returns the value of C<UV_ENODEV>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ENOENT

C<static method UV_ENOENT : int ();>

Returns the value of C<UV_ENOENT>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ENOMEM

C<static method UV_ENOMEM : int ();>

Returns the value of C<UV_ENOMEM>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ENONET

C<static method UV_ENONET : int ();>

Returns the value of C<UV_ENONET>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ENOPROTOOPT

C<static method UV_ENOPROTOOPT : int ();>

Returns the value of C<UV_ENOPROTOOPT>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ENOSPC

C<static method UV_ENOSPC : int ();>

Returns the value of C<UV_ENOSPC>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ENOSYS

C<static method UV_ENOSYS : int ();>

Returns the value of C<UV_ENOSYS>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ENOTCONN

C<static method UV_ENOTCONN : int ();>

Returns the value of C<UV_ENOTCONN>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ENOTDIR

C<static method UV_ENOTDIR : int ();>

Returns the value of C<UV_ENOTDIR>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ENOTEMPTY

C<static method UV_ENOTEMPTY : int ();>

Returns the value of C<UV_ENOTEMPTY>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ENOTSOCK

C<static method UV_ENOTSOCK : int ();>

Returns the value of C<UV_ENOTSOCK>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ENOTSUP

C<static method UV_ENOTSUP : int ();>

Returns the value of C<UV_ENOTSUP>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EPERM

C<static method UV_EPERM : int ();>

Returns the value of C<UV_EPERM>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EPIPE

C<static method UV_EPIPE : int ();>

Returns the value of C<UV_EPIPE>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EPROTO

C<static method UV_EPROTO : int ();>

Returns the value of C<UV_EPROTO>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EPROTONOSUPPORT

C<static method UV_EPROTONOSUPPORT : int ();>

Returns the value of C<UV_EPROTONOSUPPORT>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EPROTOTYPE

C<static method UV_EPROTOTYPE : int ();>

Returns the value of C<UV_EPROTOTYPE>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ERANGE

C<static method UV_ERANGE : int ();>

Returns the value of C<UV_ERANGE>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EROFS

C<static method UV_EROFS : int ();>

Returns the value of C<UV_EROFS>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ESHUTDOWN

C<static method UV_ESHUTDOWN : int ();>

Returns the value of C<UV_ESHUTDOWN>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ESPIPE

C<static method UV_ESPIPE : int ();>

Returns the value of C<UV_ESPIPE>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ESRCH

C<static method UV_ESRCH : int ();>

Returns the value of C<UV_ESRCH>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ETIMEDOUT

C<static method UV_ETIMEDOUT : int ();>

Returns the value of C<UV_ETIMEDOUT>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ETXTBSY

C<static method UV_ETXTBSY : int ();>

Returns the value of C<UV_ETXTBSY>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EXDEV

C<static method UV_EXDEV : int ();>

Returns the value of C<UV_EXDEV>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_UNKNOWN

C<static method UV_UNKNOWN : int ();>

Returns the value of C<UV_UNKNOWN>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EOF

C<static method UV_EOF : int ();>

Returns the value of C<UV_EOF>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ENXIO

C<static method UV_ENXIO : int ();>

Returns the value of C<UV_ENXIO>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_EMLINK

C<static method UV_EMLINK : int ();>

Returns the value of C<UV_EMLINK>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_ASYNC

C<static method UV_ASYNC : int ();>

Returns the value of C<UV_ASYNC>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_CHECK

C<static method UV_CHECK : int ();>

Returns the value of C<UV_CHECK>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_FS_EVENT

C<static method UV_FS_EVENT : int ();>

Returns the value of C<UV_FS_EVENT>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_FS_POLL

C<static method UV_FS_POLL : int ();>

Returns the value of C<UV_FS_POLL>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_IDLE

C<static method UV_IDLE : int ();>

Returns the value of C<UV_IDLE>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_NAMED_PIPE

C<static method UV_NAMED_PIPE : int ();>

Returns the value of C<UV_NAMED_PIPE>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_POLL

C<static method UV_POLL : int ();>

Returns the value of C<UV_POLL>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_PREPARE

C<static method UV_PREPARE : int ();>

Returns the value of C<UV_PREPARE>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_PROCESS

C<static method UV_PROCESS : int ();>

Returns the value of C<UV_PROCESS>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_STREAM

C<static method UV_STREAM : int ();>

Returns the value of C<UV_STREAM>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_TCP

C<static method UV_TCP : int ();>

Returns the value of C<UV_TCP>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_TIMER

C<static method UV_TIMER : int ();>

Returns the value of C<UV_TIMER>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_TTY

C<static method UV_TTY : int ();>

Returns the value of C<UV_TTY>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_UDP

C<static method UV_UDP : int ();>

Returns the value of C<UV_UDP>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_SIGNAL

C<static method UV_SIGNAL : int ();>

Returns the value of C<UV_SIGNAL>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_FILE

C<static method UV_FILE : int ();>

Returns the value of C<UV_FILE>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_RUN_DEFAULT

C<static method UV_RUN_DEFAULT : int ();>

Returns the value of C<UV_RUN_DEFAULT>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_RUN_ONCE

C<static method UV_RUN_ONCE : int ();>

Returns the value of C<UV_RUN_ONCE>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_RUN_NOWAIT

C<static method UV_RUN_NOWAIT : int ();>

Returns the value of C<UV_RUN_NOWAIT>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_LOOP_BLOCK_SIGNAL

C<static method UV_LOOP_BLOCK_SIGNAL : int ();>

Returns the value of C<UV_LOOP_BLOCK_SIGNAL>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_READABLE

C<static method UV_READABLE : int ();>

Returns the value of C<UV_READABLE>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_WRITABLE

C<static method UV_WRITABLE : int ();>

Returns the value of C<UV_WRITABLE>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_DISCONNECT

C<static method UV_DISCONNECT : int ();>

Returns the value of C<UV_DISCONNECT>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_PRIORITIZED

C<static method UV_PRIORITIZED : int ();>

Returns the value of C<UV_PRIORITIZED>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_TTY_MODE_NORMAL

C<static method UV_TTY_MODE_NORMAL : int ();>

Returns the value of C<UV_TTY_MODE_NORMAL>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_TTY_MODE_RAW

C<static method UV_TTY_MODE_RAW : int ();>

Returns the value of C<UV_TTY_MODE_RAW>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_TTY_MODE_IO

C<static method UV_TTY_MODE_IO : int ();>

Returns the value of C<UV_TTY_MODE_IO>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_JOIN_GROUP

C<static method UV_JOIN_GROUP : int ();>

Returns the value of C<UV_JOIN_GROUP>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

=head2 UV_LEAVE_GROUP

C<static method UV_LEAVE_GROUP : int ();>

Returns the value of C<UV_LEAVE_GROUP>. If this constant is not defined on the system, L<Error::NotSupported|SPVM::Error::NotSupported> exception is thrown.

